@@ -6,7 +6,7 @@ from forms import GeneralModelConverter
 from .security.decorators import has_access
 from .filemanager import uuid_originalname
 from .widgets import FormWidget, ShowWidget, ListWidget, SearchWidget
-
+from .actions import ActionItem
 
 
 def expose(url='/', methods=('GET',)):
@@ -36,9 +36,9 @@ class BaseView(object):
     static_folder='static'
     
     base_permissions = []
-    redirect_url = '/'
-
     actions = []
+    
+    default_view = 'list'
 
     def __init__(self):
         """
@@ -57,7 +57,7 @@ class BaseView(object):
                     self.base_permissions.append(action.name)
                     self.actions.append(action)
             
-            
+
 
     def create_blueprint(self, baseapp, 
                         endpoint = None, 
@@ -70,7 +70,10 @@ class BaseView(object):
 
         # If endpoint name is not provided, get it from the class name
         if not self.endpoint:
-            self.endpoint = self.__class__.__name__.lower()
+            self.endpoint = self.__class__.__name__
+
+        if self.route_base is None:
+            self.route_base = '/' + self.__class__.__name__.lower()
 
         self.static_folder = static_folder
         if not static_folder:
@@ -100,32 +103,81 @@ class BaseView(object):
         
 
     def _prettify_name(self, name):
+        """
+        Prettify pythonic variable name.
+
+        For example, 'hello_world' will be converted to 'Hello World'
+
+        :param name:
+            Name to prettify
+        """
         return re.sub(r'(?<=.)([A-Z])', r' \1', name)
 
+    def _prettify_column(self, name):
+        """
+        Prettify pythonic variable name.
+
+        For example, 'hello_world' will be converted to 'Hello World'
+
+        :param name:
+            Name to prettify
+        """
+        return name.replace('_', ' ').title()
+
 
     def _get_redirect(self):
         if (request.args.get('next')):
             return request.args.get('next')
         else:
-            return self.redirect_url
+            try:
+                return url_for('%s.%s' % (self.endpoint, self.default_view))
+            except:
+                return '/'
+            
 
     def _get_group_by_args(self):
         group_by = request.args.get('group_by')
         if not group_by: group_by = ''
         return group_by
 
-    def _get_order_args(self):
-        order_column = request.args.get('order_column')
-        order_direction = request.args.get('order_direction')
-        if not order_column: order_column = ''
-        if not order_direction: order_direction = ''
-        return order_column, order_direction
+    def _get_page_args(self, pages = {}):
+        """
+        Get page arguments, return a dictionary
+        { <VIEW_NAME>: PAGE_NUMBER }
+        
+        Arguments are passed: page_<VIEW_NAME>=<PAGE_NUMBER>
+        
+        """
+        for arg in request.args:
+            re_match = re.findall('page_(.*)', arg)
+            if re_match:
+                pages[re_match[0]] = int(request.args.get(arg))
+        return pages
+
+
+
+    def _get_order_args(self, orders = {}):
+        """
+        Get order arguments, return a dictionary
+        { <VIEW_NAME>: (ORDER_COL, ORDER_DIRECTION) }
+        
+        Arguments are passed like: _oc_<VIEW_NAME>=<COL_NAME>&_od_<VIEW_NAME>='asc'|'desc'
+        
+        """
+        for arg in request.args:
+            re_match = re.findall('_oc_(.*)', arg)
+            if re_match:
+                orders[re_match[0]] = (request.args.get(arg),request.args.get('_od_' + re_match[0]))
+        return orders
+                
 
     def _get_filter_args(self, filters={}):
-        for n in request.args:
-            re_match = re.findall('_flt_(.*)', n)
+        for arg in request.args:
+            re_match = re.findall('_flt_(.*)', arg)
             if re_match:
-                filters[re_match[0]] = request.args.get(n)
+                # ignore select2 __None value
+                if request.args.get(arg) not in ('__None',''):
+                    filters[re_match[0]] = request.args.get(arg)
         return filters
 
 
@@ -171,8 +223,7 @@ class SimpleFormView(BaseView):
     form_title = 'Form Title'
     form_columns = []
     form = None
-    redirect_url = '/'
-
+    
     @expose("/form", methods=['GET'])
     @has_access
     def this_form_get(self):
@@ -208,6 +259,9 @@ class SimpleFormView(BaseView):
                     )
 
     def form_post(self, form):
+        """
+        Override this method to implement your form processing
+        """
         pass
 
     def _get_edit_widget(self, form = None, exclude_cols = [], widgets = {}):
@@ -220,11 +274,10 @@ class SimpleFormView(BaseView):
 
 
 class BaseCRUDView(BaseView):
-    route_base = '/general'
+    
     datamodel = None
     related_views = []
-    redirect_url = route_base + '/list'
-
+    
     """ Titles """
     list_title = ""
     show_title = ""
@@ -241,6 +294,8 @@ class BaseCRUDView(BaseView):
 
     label_columns = {}
     description_columns = {}
+    
+    page_size = 30
 
     """ fieldsets [(<'TITLE'|None>, {'fields':[<F1>,<F2>,...]}),....] """
     show_fieldsets = []
@@ -291,26 +346,44 @@ class BaseCRUDView(BaseView):
                     self.validators_columns,
                     self.search_columns)
 
+    def _init_titles(self):
+        if not self.list_title:
+            self.list_title = 'List ' + self._prettify_name(self.datamodel.obj.__name__)
+        if not self.add_title:
+            self.add_title = 'Add ' + self._prettify_name(self.datamodel.obj.__name__)
+        if not self.edit_title:
+            self.edit_title = 'Edit ' + self._prettify_name(self.datamodel.obj.__name__)
+        if not self.show_title:
+            self.show_title = 'Show ' + self._prettify_name(self.datamodel.obj.__name__)
 
     def _init_vars(self):
+        self._init_titles()
+            
+        list_cols = self.datamodel.get_columns_list()
+        for col in list_cols:
+            if not self.label_columns.get(col):
+                self.label_columns[col] = self._prettify_column(col)
         if self.show_fieldsets:
             self.show_columns = []
             for fieldset_item in self.show_fieldsets:                
-                self.show_columns = self.show_columns + list(fieldset_item[1].get('fields'))                
+                self.show_columns = self.show_columns + list(fieldset_item[1].get('fields'))
+        else:
+            if not self.show_columns:
+                self.show_columns = list_cols
         if self.add_fieldsets:
             self.add_columns = []
             for fieldset_item in self.add_fieldsets:
                 self.add_columns = self.add_columns + list(fieldset_item[1].get('fields'))
         else:
             if not self.add_columns:
-                self.add_columns = self.datamodel.get_columns_list()
+                self.add_columns = list_cols
         if self.edit_fieldsets:
             self.edit_columns = []
             for fieldset_item in self.edit_fieldsets:
                 self.edit_columns = self.edit_columns + list(fieldset_item[1].get('fields'))
         else:
             if not self.edit_columns:
-                self.edit_columns = self.datamodel.get_columns_list()
+                self.edit_columns = list_cols
                         
         
     def __init__(self, **kwargs):
@@ -325,29 +398,42 @@ class BaseCRUDView(BaseView):
     --------------------------
     """
     def _get_related_list_widget(self, item, related_view, 
-                                filters={}, order_column='', order_direction=''):
+                                filters={}, order_column='', order_direction='',
+                                page=None, page_size=None):
 
         fk = related_view.datamodel.get_related_fk(self.datamodel.obj)
         filters[fk] = item
         return related_view._get_list_widget(filters = filters, 
-                    order_column = order_column, order_direction = order_direction)
+                    order_column = order_column, order_direction = order_direction, page=page, page_size=page_size)
         
-    def _get_related_list_widgets(self, item, filters = {}, order_column='', order_direction='', 
-                                widgets = {}, **args):
+    def _get_related_list_widgets(self, item, filters = {}, orders = {}, 
+                                pages=None, widgets = {}, **args):
         widgets['related_lists'] = []
         for view in self.related_views:
-            widgets['related_lists'].append(self._get_related_list_widget(item, view, filters, order_column, order_direction).get('list'))
+            if orders.get(view.__class__.__name__):
+                order_column, order_direction = orders.get(view.__class__.__name__)
+            else: order_column, order_direction = '',''
+            widgets['related_lists'].append(self._get_related_list_widget(item, view, filters, 
+                    order_column, order_direction, page=pages.get(view.__class__.__name__), page_size=view.page_size).get('list'))
         return widgets
     
-    def _get_list_widget(self, filters={}, order_column='', order_direction='', widgets = {}, **args):
+    def _get_list_widget(self, filters = {}, 
+                        order_column = '', 
+                        order_direction = '',
+                        page = None,
+                        page_size = None,
+                        widgets = {}, **args):
 
-        lst = self.datamodel.query(filters, order_column, order_direction)
+        count, lst = self.datamodel.query(filters, order_column, order_direction, page=page, page_size=page_size)
         pks = self.datamodel.get_keys(lst)
         widgets['list'] = self.list_widget(route_base = self.route_base,
                                                 label_columns = self.label_columns,
                                                 include_columns = self.list_columns,
                                                 value_columns = self.datamodel.get_values(lst, self.list_columns),
                                                 order_columns = self.order_columns,
+                                                page = page,
+                                                page_size = page_size,
+                                                count = count,
                                                 pks = pks,
                                                 filters = filters,
                                                 generalview_name = self.__class__.__name__
@@ -366,7 +452,6 @@ class BaseCRUDView(BaseView):
         if show_additional_links:
             additional_links = show_additional_links
         else: additional_links = self.show_additional_links
-        print "ADDLINKS", additional_links
         item = self.datamodel.get(id)
         widgets['show'] = self.show_widget(route_base = self.route_base,
                                                 pk = id,
@@ -454,32 +539,38 @@ class GeneralView(BaseCRUDView):
     --------------------------------
     """
     
-    @expose('/list/', methods=['GET', 'POST'])
+    @expose('/list/')
     @has_access
     def list(self):
 
         form = self.search_form.refresh()
-        search_form = self.search_form(request.form)
-
-        order_column, order_direction = self._get_order_args()
+        
+        if self._get_order_args().get(self.__class__.__name__):
+            order_column, order_direction = self._get_order_args().get(self.__class__.__name__)
+        else: order_column, order_direction = '',''
+        page = self._get_page_args().get(self.__class__.__name__)
 
         filters = {}
         filters = self._get_filter_args(filters)
-        filters = self._get_dict_from_form(search_form, filters)
         if (filters != {}):
             item = self.datamodel.obj()
             for filter_key in filters:
+                # on related models translate id to model obj
                 try:
                     rel_obj = self.datamodel.get_related_obj(filter_key, filters.get(filter_key))
                     setattr(item, filter_key, rel_obj)
-                    search_form = self.search_form(obj = item)
+                    form = self.search_form(obj = item)
                     filters[filter_key] = rel_obj
                 except:
                     setattr(item, filter_key, filters.get(filter_key))
-                    search_form = self.search_form(obj = item)
+                    form = self.search_form(obj = item)
 
-        widgets = self._get_list_widget(filters, order_column, order_direction)
-        widgets = self._get_search_widget(form = search_form, widgets = widgets)
+        widgets = self._get_list_widget(filters = filters, 
+                    order_column = order_column, 
+                    order_direction = order_direction, 
+                    page = page, 
+                    page_size = self.page_size)
+        widgets = self._get_search_widget(form = form, widgets = widgets)
 
         return render_template(self.list_template,
                                         title = self.list_title,
@@ -493,14 +584,19 @@ class GeneralView(BaseCRUDView):
             SHOW
     --------------------------------
     """
-    @expose('/show/<int:id>', methods=['GET'])
+    @expose('/show/<int:pk>', methods=['GET'])
     @has_access
-    def show(self, id):
+    def show(self, pk):
 
-        widgets = self._get_show_widget(id)
-        item = self.datamodel.get(id)
-        widgets = self._get_related_list_widgets(item, filters = {}, order_column='', order_direction='', widgets = widgets)
+        widgets = self._get_show_widget(pk)
+        item = self.datamodel.get(pk)
+        pages = self._get_page_args()
+        orders = self._get_order_args()
+        
+        widgets = self._get_related_list_widgets(item, filters = {}, orders = orders, 
+                pages = pages, widgets = widgets)
         return render_template(self.show_template,
+                           pk = pk,
                            title = self.show_title,
                            widgets = widgets,
                            baseapp = self.baseapp,
@@ -551,6 +647,9 @@ class GeneralView(BaseCRUDView):
     @has_access
     def edit(self, pk = 0):
 
+        pages = self._get_page_args()
+        orders = self._get_order_args()
+        
         item = self.datamodel.get(pk)
         filters = self._get_filter_args(filters={})
         exclude_cols = self.datamodel.get_relation_filters(filters)
@@ -571,7 +670,8 @@ class GeneralView(BaseCRUDView):
                 return redirect(self._get_redirect())
             else:
                 widgets = self._get_edit_widget(form = form, exclude_cols = exclude_cols)
-                widgets = self._get_related_list_widgets(item, filters = {}, order_column='', order_direction='', widgets = widgets)
+                widgets = self._get_related_list_widgets(item, filters = {}, 
+                        orders = orders, pages = pages, widgets = widgets)
                 return render_template(self.edit_template,
                         title = self.edit_title,
                         widgets = widgets,
@@ -581,7 +681,8 @@ class GeneralView(BaseCRUDView):
             form = self.edit_form(obj=item)
             form = form.refresh(obj=item)
             widgets = self._get_edit_widget(form = form, exclude_cols = exclude_cols)
-            widgets = self._get_related_list_widgets(item, filters = {}, order_column='', order_direction='', widgets = widgets)
+            widgets = self._get_related_list_widgets(item, filters = {}, 
+                        orders = orders, pages = pages, widgets = widgets)                
             return render_template(self.edit_template,
                             title = self.edit_title,
                             widgets = widgets,
@@ -594,10 +695,10 @@ class GeneralView(BaseCRUDView):
             DELETE
     ---------------------------
     """
-    @expose('/delete/<int:id>')
+    @expose('/delete/<int:pk>')
     @has_access
-    def delete(self, id):
-        item = self.datamodel.get(id)
+    def delete(self, pk):
+        item = self.datamodel.get(pk)
         
         self.pre_delete(item)
         self.datamodel.delete(item)
